#! /usr/bin/env python

"""
Convert PWM outputs from a ULog file into an RCbencemark script

This script depends on https://github.com/PX4/pyulog (which depends on numpy)
"""

from __future__ import print_function
import argparse
from pyulog import ULog


js_template = '''/* //////////////// generated by ulog2rcbm ////////////////
- filePrefix
- data -- [[timestamp_in_ms, pwm_output], [..., ...], ...]
- samplesAvg
///////////// User defined variables //////////// */

const filePrefix = {};
const data = {};
const samplesAvg = {};

let lastTimestamp = data[0][0];
let position = 0; // index of data array
let timeDifference;

///////////////// Beginning of the script //////////////////

if (!data) {{
    rcb.console.print("Error: data is not an array of length > 0");
    rcb.endScript();
}}

rcb.console.setVerbose(false);

rcb.files.newLogFile({{prefix: filePrefix}});

readSensor();

function readSensor() {{
    rcb.sensors.read(readDone, samplesAvg);
}}
function readDone(result) {{
    rcb.files.newLogEntry(result, readSensor);
}}

rcb.console.print("Starting...");

rcb.wait(callback, lastTimestamp > 5 ? 5 : lastTimestamp); // wait 5s or first timestamp, whichever smaller

function callback() {{
    rcb.output.set("esc", data[position][1]);
    position++;
    if (data.length > position) {{
        timeDifference = data[position][0] - lastTimestamp;
        lastTimestamp = data[position][0];
        rcb.wait(callback, timeDifference);
    }} else {{
        rcb.wait(function() {{
            rcb.endScript();
        }}, 1);
    }}
}}
'''

def main():
    parser = argparse.ArgumentParser(description='Convert ULog to RCbenchmark script')
    parser.add_argument('filename', metavar='file.ulg', help='ULog input file')
    parser.add_argument('-m', '--multi', dest='multi', action='store',
                        help="multi_id (which actuator_outputs to use, '0' or '1', default is '0')", default='0')
    parser.add_argument('-a', '--actuator', dest='actuator', action='store',
                        help="which actuator's output to use ('0' - '15', default is '0')", default='0')
    parser.add_argument('--avg', dest='average', action='store',
                        help="sample average (default is '1')", default='1')
    args = parser.parse_args()
    convert_ulog2rcbm(args.filename, args.multi, args.actuator, args.average)

def convert_ulog2rcbm(ulog_file_name, multi_id, actuator, sample_average):
    multi_id = int(multi_id)
    sample_average = int(sample_average)
    ulog = ULog(ulog_file_name, 'actuator_outputs', False)
    for d in ulog.data_list:
        if d.multi_id == multi_id:
            data = d
            break
    output_file_prefix = ulog_file_name
    if output_file_prefix.lower().endswith('.ulg'):
        output_file_prefix = output_file_prefix[:-4]
    output_file_prefix = '{0}_{1}_{2}_{3}'.format(output_file_prefix, d.name, d.multi_id, actuator)
    with open(output_file_prefix + '.js', 'w') as js_file:
        timestamp_list = data.data['timestamp']
        pwm_list = data.data['output[{}]'.format(actuator)]
        data_string = '[{}]'.format(
            ','.join(
                [ '[{},{}]'.format(row[0] / 1000000, int(row[1])) for row in list(zip(timestamp_list, pwm_list)) ]
            )
        )

        if output_file_prefix[:2] == './' or output_file_prefix[:2] == '.\\':
            output_file_prefix = output_file_prefix[2:]


        js_file.write(js_template.format('"{}"'.format(output_file_prefix), data_string, sample_average))

if __name__ == '__main__':
    main()



#! /usr/bin/env python

"""
Generate an RCbencemark script that outputs a sin wave
"""

from __future__ import print_function
import argparse
import math


js_template = '''/* //////////////// generated by ulog2rcbm ////////////////
- filePrefix
- data -- [[timestamp_in_ms, pwm_output], [..., ...], ...]
- samplesAvg
- rampup
///////////// User defined variables //////////// */

const filePrefix = {};
const data = {};
const samplesAvg = {};
const rampup = {};

let lastTimestamp = data[0][0];
let position = 0; // index of data array
let timeDifference;

///////////////// Beginning of the script //////////////////

if (!data) {{
    rcb.console.print("Error: data is not an array of length > 0");
    rcb.endScript();
}}

rcb.console.setVerbose(false);

rcb.files.newLogFile({{prefix: filePrefix}});

readSensor();

function readSensor() {{
    rcb.sensors.read(readDone, samplesAvg);
}}
function readDone(result) {{
    rcb.files.newLogEntry(result, readSensor);
}}

rcb.console.print("Starting...");

if (rampup) {{
    rcb.console.print("Ramping Up...");
    rcb.output.ramp("esc", 1000, data[0][1], lastTimestamp > 5 ? lastTimestamp : 5, callback); // ramp up to data[0][1] in data[0][0] or 5 sec, whichever is larger
}} else {{
    if (lastTimestamp > 0) {{
        rcb.wait(callback, lastTimestamp);
    }} else {{
        callback();
    }}
}}

function callback() {{
    rcb.output.set("esc", data[position][1]);
    position++;
    if (data.length > position) {{
        timeDifference = data[position][0] - lastTimestamp;
        lastTimestamp = data[position][0];
        rcb.wait(callback, timeDifference);
    }} else {{
        rcb.wait(function() {{
            rcb.endScript();
        }}, 1);
    }}
}}
'''

def main():
    parser = argparse.ArgumentParser(description='Generate a RCbenchmark script that produces sinusoidal wave')
    parser.add_argument('-o', '--offset', dest='offset', action='store',
                        help="offset of the wave ('1000' - '2000', default is '1500')", default='1500')
    parser.add_argument('-a', '--amplitude', dest='amplitude', action='store',
                        help="amplitude, the peak deviation of the wave from offset ('0' - '500', default is '500')", default='500')
    parser.add_argument('-f', '--frequency', dest='frequency', action='store',
                        help="angular frequency in radians per second (default is '1')", default='1')
    parser.add_argument('-p', '--phase', dest='phase', action='store',
                        help="phase, specifies in radians where in its cycle the oscillation is at t=0 (default is '0')", default='0')
    parser.add_argument('-r', '--rampup', dest='rampup', action='store_true',
                        help="ramp up from 1000 to initial value (disabled by default)", default=False)
    parser.add_argument('--avg', dest='average', action='store',
                        help="sample average (default is '1')", default='1')
    args = parser.parse_args()
    gen_script(args.offset, args.amplitude, args.frequency, args.phase, args.rampup, args.average)

def gen_script(offset, amplitude, omega, phi, rampup, sample_average):
    offset = int(float(offset))
    assert offset >= 1000 and offset <= 2000
    amplitude = int(float(amplitude))
    assert amplitude >= 0 and amplitude <= 500
    omega = float(omega)
    phi = float(phi)
    sample_average = int(sample_average)
    output_file_prefix = 'sin_wave_{}_{}_{}_{}_{}'.format(offset, amplitude, omega, phi, sample_average)
    data = [ [ timestamp, offset + amplitude * math.sin( timestamp / 1000 * omega + phi ) ] for timestamp in range(0, 1000 * 40, 25) ] # ms
    
    with open(output_file_prefix + '.js', 'w') as js_file:
        data_string = '[{}]'.format(
            ','.join(
                [ '[{},{}]'.format(row[0] / 1000, int(row[1])) for row in data ]
            )
        )
        js_file.write(js_template.format('"{}"'.format(output_file_prefix), data_string, sample_average, 'true' if rampup else 'false'))
    return output_file_prefix + '.js'

if __name__ == '__main__':
    main()



#! /usr/bin/env python

import os
from distutils import dir_util
import pytest
from pyrcbm import sin_rcbm

@pytest.fixture
def datadir(tmpdir, request):
    '''
    Fixture responsible for searching a folder with the same name of test
    module and, if available, moving all contents to a temporary directory so
    tests can use them freely.
    '''
    filename = request.module.__file__
    test_dir, _ = os.path.splitext(filename)

    if os.path.isdir(test_dir):
        dir_util.copy_tree(test_dir, str(tmpdir))

    return tmpdir

class TestSinRcbm:
    def check_output_file(self, tmpdir, expected_output_file, test):
        if expected_output_file:
            print('getting output -- CWD: {}'.format(os.getcwd()))
            expected_file = open(expected_output_file, 'r')
            expected_file_content = expected_file.read()
            expected_file.close()
        else:
            expected_file_content = ''

        os.chdir(tmpdir)
        print('tmpdir -- CWD: {}'.format(os.getcwd()))
        filename = sin_rcbm.gen_script(test['offset'], test['amplitude'], test['omega'], test['phi'], test['rampup'], test['sample_average'])
        with open(filename, 'r') as output_file:
            output_file_content = output_file.read()
            assert output_file_content == expected_file_content

    def test_default(self, datadir):
        test = {
            'offset': '1500',
            'amplitude': '500',
            'omega': '1',
            'phi': '0',
            'rampup': False,
            'sample_average': '1',
        }
        self.check_output_file(datadir, datadir.join('sin_default.js'), test)

    def test_offset_in_range_1(self, datadir):
        test = {
            'offset': '1000',
            'amplitude': '500',
            'omega': '1',
            'phi': '0',
            'rampup': False,
            'sample_average': '1',
        }
        self.check_output_file(datadir, datadir.join('sin_offset_in_range_1.js'), test)

    def test_offset_in_range_2(self, datadir):
        test = {
            'offset': '2000',
            'amplitude': '500',
            'omega': '1',
            'phi': '0',
            'rampup': False,
            'sample_average': '1',
        }
        self.check_output_file(datadir, datadir.join('sin_offset_in_range_2.js'), test)

    def test_offset_out_of_range_1(self, datadir):
        test = {
            'offset': '999',
            'amplitude': '500',
            'omega': '1',
            'phi': '0',
            'rampup': False,
            'sample_average': '1',
        }
        with pytest.raises(AssertionError):
            self.check_output_file(datadir, '', test)

    def test_offset_out_of_range_2(self, datadir):
        test = {
            'offset': '2001',
            'amplitude': '500',
            'omega': '1',
            'phi': '0',
            'rampup': False,
            'sample_average': '1',
        }
        with pytest.raises(AssertionError):
            self.check_output_file(datadir, '', test)

    def test_amplitude_in_range(self, datadir):
        test = {
            'offset': '1500',
            'amplitude': '0',
            'omega': '1',
            'phi': '0',
            'rampup': False,
            'sample_average': '1',
        }
        self.check_output_file(datadir, datadir.join('sin_amplitude_in_range.js'), test)

    def test_amplitude_out_of_range_1(self, datadir):
        test = {
            'offset': '1500',
            'amplitude': '-1',
            'omega': '1',
            'phi': '0',
            'rampup': False,
            'sample_average': '1',
        }
        with pytest.raises(AssertionError):
            self.check_output_file(datadir, '', test)

    def test_amplitude_out_of_range_2(self, datadir):
        test = {
            'offset': '1500',
            'amplitude': '501',
            'omega': '1',
            'phi': '0',
            'rampup': False,
            'sample_average': '1',
        }
        with pytest.raises(AssertionError):
            self.check_output_file(datadir, '', test)

    def test_frequency_1(self, datadir):
        test = {
            'offset': '1500',
            'amplitude': '500',
            'omega': '0.1',
            'phi': '0',
            'rampup': False,
            'sample_average': '1',
        }
        self.check_output_file(datadir, datadir.join('sin_frequency_1.js'), test)

    def test_frequency_2(self, datadir):
        test = {
            'offset': '1500',
            'amplitude': '500',
            'omega': '99',
            'phi': '0',
            'rampup': False,
            'sample_average': '1',
        }
        self.check_output_file(datadir, datadir.join('sin_frequency_2.js'), test)

    def test_phase_1(self, datadir):
        test = {
            'offset': '1500',
            'amplitude': '500',
            'omega': '1',
            'phi': '6.28',
            'rampup': False,
            'sample_average': '1',
        }
        self.check_output_file(datadir, datadir.join('sin_phase_1.js'), test)

    def test_phase_2(self, datadir):
        test = {
            'offset': '1500',
            'amplitude': '500',
            'omega': '1',
            'phi': '-5',
            'rampup': False,
            'sample_average': '1',
        }
        self.check_output_file(datadir, datadir.join('sin_phase_2.js'), test)

    def test_rampup(self, datadir):
        test = {
            'offset': '1500',
            'amplitude': '500',
            'omega': '1',
            'phi': '0',
            'rampup': True,
            'sample_average': '1',
        }
        self.check_output_file(datadir, datadir.join('sin_rampup.js'), test)

    def test_avg(self, datadir):
        test = {
            'offset': '1500',
            'amplitude': '500',
            'omega': '1',
            'phi': '0',
            'rampup': False,
            'sample_average': '100',
        }
        self.check_output_file(datadir, datadir.join('sin_avg.js'), test)



#! /usr/bin/env python

import os
from distutils import dir_util
import pytest
from pyrcbm import ulog2rcbm

@pytest.fixture
def datadir(tmpdir, request):
    '''
    Fixture responsible for searching a folder with the same name of test
    module and, if available, moving all contents to a temporary directory so
    tests can use them freely.
    '''
    filename = request.module.__file__
    test_dir, _ = os.path.splitext(filename)

    if os.path.isdir(test_dir):
        dir_util.copy_tree(test_dir, str(tmpdir))

    return tmpdir

class TestSinRcbm:
    def check_output_file(self, tmpdir, expected_output_file, test):
        if expected_output_file:
            print('getting output -- CWD: {}'.format(os.getcwd()))
            expected_file = open(expected_output_file, 'r')
            expected_file_content = expected_file.read()
            expected_file.close()
        else:
            expected_file_content = ''

        os.chdir(tmpdir)
        print('tmpdir -- CWD: {}'.format(os.getcwd()))
        ulog2rcbm.convert_ulog2rcbm(test['filename'], test['multi'], test['actuator'], test['average'])

        output_file_prefix = test['filename']
        if output_file_prefix.lower().endswith('.ulg'):
            output_file_prefix = output_file_prefix[:-4]

        with open('{0}_{1}_{2}_{3}.js'.format(output_file_prefix, 'actuator_outputs', test['multi'], test['actuator']), 'r') as output_file:
            output_file_content = output_file.read()
            assert output_file_content == expected_file_content

    def test_default(self, datadir):
        test = {
            'filename': 'sample1.ulg',
            'multi': '0',
            'actuator': '0',
            'average': '1',
        }
        self.check_output_file(datadir, datadir.join('ulog_default.js'), test)

    def test_actuator_1(self, datadir):
        test = {
            'filename': 'sample1.ulg',
            'multi': '0',
            'actuator': '3',
            'average': '1',
        }
        self.check_output_file(datadir, datadir.join('ulog_actuator_1.js'), test)

    def test_actuator_2(self, datadir):
        test = {
            'filename': 'sample1.ulg',
            'multi': '0',
            'actuator': '15',
            'average': '1',
        }
        self.check_output_file(datadir, datadir.join('ulog_actuator_2.js'), test)

    def test_avg(self, datadir):
        test = {
            'filename': 'sample1.ulg',
            'multi': '0',
            'actuator': '0',
            'average': '100',
        }
        self.check_output_file(datadir, datadir.join('ulog_avg.js'), test)
